//  Original pkgutils:
//  Copyright (c) 2000-2005 Per Liden
//
//  That C-rewrite:
//  Copyright (c) 2006 Anton Vorontsov <cbou@mail.ru>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
//  USA.
//

#pragma once
#include <sys/types.h>
#include <stdlib.h>
#include <pkgutils/list.h>
#include <pkgutils/misc.h>

#define PKG_EXT         ".pkg.tar.gz"
#define PKG_DB_DIR      "var/lib/pkg/"
#define PKG_REJECT_DIR  "var/lib/pkg/rejected/"
#define PKG_ADD_CONFIG  "usr/etc/pkgadd.conf"

#define PKG_ADD_FORCE      1
#define PKG_ADD_FORCE_PERM 2

typedef struct {
	char *path;
	mode_t mode;
	uid_t uid;
	gid_t gid;
} pkg_file_t;

typedef struct {
	char *name;
	char *version;
	list_t files;  // pkg_files_t list
} pkg_desc_t;

typedef enum {
	CONFLICT_NONE = 0,  // gag
	CONFLICT_LINK = 1,  // pseudo conflict
	CONFLICT_SELF = 2,  // pseudo conflict when upgrading
	CONFLICT_REF  = 4,  // pseudo conflict when upgrading
	CONFLICT_DB   = 8,  // file conflict in database
	CONFLICT_FS   = 16, // file conflict in filesystem
	CONFLICT_PERM = 32  // mode/ownership conflict
} pkg_conflict_type_t;

typedef struct {
	pkg_desc_t *pkg;
	pkg_file_t *file;
	pkg_conflict_type_t type;
} pkg_conflict_t;

char *opt_root;
list_t pkg_db;
int db_lock;

void pkg_init_db();
void pkg_free_db();
int pkg_commit_db();

list_t *pkg_list_files(const char *pkg_name);
list_t *pkg_list_installed();

int pkg_add(const char *pkg_path, int opts);
//int pkg_rm(const char *name);

#ifdef DEBUG
#define dbg(str, ...) printf(str, __VA_ARGS__)
#else
#define dbg(str, ...)
#endif
